/**
 * 
 */
package jalview.ext.jmol;

import jalview.api.FeatureRenderer;
import jalview.api.SequenceRenderer;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.SequenceI;
import jalview.structure.StructureMapping;
import jalview.structure.StructureSelectionManager;
import jalview.util.Comparison;

import java.awt.Color;
import java.util.ArrayList;

/**
 * Routines for generating Jmol commands for Jalview/Jmol binding
 * @author JimP
 *
 */
public class JmolCommands
{

  /**
   * Jmol utility which constructs the commands to colour chains by the given alignment
   * 
   */
  public static String[] getColourBySequenceCommand(StructureSelectionManager ssm, String[] files, SequenceI[][] sequence, SequenceRenderer sr, FeatureRenderer fr, AlignmentI alignment)
  {
    ArrayList<String> str = new ArrayList<String>();
    StringBuffer command = new StringBuffer();
  
    for (int pdbfnum = 0; pdbfnum < files.length; pdbfnum++)
    {
      StructureMapping[] mapping = ssm.getMapping(files[pdbfnum]);
  
      if (mapping == null || mapping.length < 1)
        continue;
  
      int lastPos = -1;
      for (int s = 0; s < sequence[pdbfnum].length; s++)
      {
        for (int sp, m = 0; m < mapping.length; m++)
        {
          if (mapping[m].getSequence() == sequence[pdbfnum][s]
                  && (sp = alignment.findIndex(sequence[pdbfnum][s])) > -1)
          {
            SequenceI asp = alignment.getSequenceAt(sp);
            for (int r = 0; r < asp.getLength(); r++)
            {
              // no mapping to gaps in sequence
              if (jalview.util.Comparison.isGap(asp.getCharAt(r)))
              {
                continue;
              }
              int pos = mapping[m].getPDBResNum(asp.findPosition(r));
  
              if (pos < 1 || pos == lastPos)
                continue;
  
              lastPos = pos;
  
              Color col = sr.getResidueBoxColour(sequence[pdbfnum][s], r);
  
              if (fr != null)
                col = fr.findFeatureColour(col, sequence[pdbfnum][s], r);
              String newSelcom = (mapping[m].getChain() != " " ? ":"
                      + mapping[m].getChain() : "")
                      + "/"
                      + (pdbfnum + 1)
                      + ".1"
                      + ";color["
                      + col.getRed()
                      + ","
                      + col.getGreen()
                      + ","
                      + col.getBlue() + "]";
              if (command.length()>newSelcom.length() && command.substring(command.length()-newSelcom.length()).equals(newSelcom))
              {
                command = JmolCommands.condenseCommand(command, pos);
                continue;
              }
              // TODO: deal with case when buffer is too large for Jmol to parse
              // - execute command and flush
  
              command.append(";");
              if (command.length()>51200)
              {
                // add another chunk
                str.add(command.toString());
                command.setLength(0);
              }
              command.append("select " + pos);
              command.append(newSelcom);
            }
            break;
          }
        }
      }
    }
    return str.toArray(new String[str.size()]);
  }

  public static StringBuffer condenseCommand(StringBuffer command, int pos)
  {
  
    // work back to last 'select'
    int p=command.length(),q=p;
    do {
      p-=6;
      if (p<1) { p=0; };
    } while ((q=command.indexOf("select",p))==-1 && p>0);
    
    StringBuffer sb = new StringBuffer(command.substring(0,q+7));
  
    command =  command.delete(0,q+7);
  
    String start;
  
    if (command.indexOf("-") > -1)
    {
      start = command.substring(0, command.indexOf("-"));
    }
    else
    {
      start = command.substring(0, command.indexOf(":"));
    }
  
    sb.append(start + "-" + pos + command.substring(command.indexOf(":")));
  
    return sb;
  }

}
